import java.io.File;
import java.io.IOException;
import java.util.Date;

import com.sun.star.beans.PropertyValue;
import com.sun.star.comp.beans.LocalOfficeConnection;
import com.sun.star.frame.XComponentLoader;
import com.sun.star.frame.XFrame;
import com.sun.star.frame.XModel;
import com.sun.star.frame.XStorable;
import com.sun.star.lang.XComponent;
import com.sun.star.uno.UnoRuntime;
import com.sun.star.uno.XComponentContext;

/**
 * Repeatedly saves a document as PDF by controlling open office.
 * 
 * @author <a href="mailto:bsneade@iogen.com">Brad Sneade</a> (NGC / Iogen)
 */
public class TestSaveAsPDF {

    private LocalOfficeConnection localOfficeConnection;

    private XComponentContext xComponentContext;

    public TestSaveAsPDF(final File openOfficePath) throws Exception {
        localOfficeConnection = new LocalOfficeConnection();
        localOfficeConnection
                .setUnoUrl("uno:socket,host=localhost,port=2002;urp;StarOffice.ServiceManager");
        xComponentContext = localOfficeConnection.getComponentContext(); // just to start OOo
        
        // I know this is probably naughty in a constructor
        log("Waiting for OOo to startup");
        Thread.sleep(1000 * 10); // wait 10 sec for OOo to get started
    } // end TestSaveAsPDF(File)

    /**
     * Invokes open office and saves the document as pdf.
     * 
     * @param documentPath
     * @param expectedSize
     * @return
     * @throws Exception
     */
    public File saveDocumentAsPDF(final String documentPath,
            final Integer expectedSize) throws Exception {
        log("Opening new document: " + documentPath);

        // retrieve the Desktop object, we need its XComponentLoader
        final Object desktop = xComponentContext.getServiceManager()
                .createInstanceWithContext("com.sun.star.frame.Desktop",
                        xComponentContext);
        final XComponentLoader xComponentLoader = (XComponentLoader) UnoRuntime
                .queryInterface(XComponentLoader.class, desktop);

        // define load properties according to
        // com.sun.star.document.MediaDescriptor
        // the boolean property AsTemplate tells the office to create a new
        // document
        // from the given file
        final PropertyValue[] loadProps = new PropertyValue[1];
        loadProps[0] = new PropertyValue();
        loadProps[0].Name = "AsTemplate";
        loadProps[0].Value = new Boolean(false); // from template
        // load the document

        final XComponent documentXComponent;
        try {
            documentXComponent = xComponentLoader.loadComponentFromURL(
                    "file:///" + documentPath, "_blank", 0, loadProps);
        } catch (com.sun.star.lang.IllegalArgumentException e) {
            throw new Exception(documentPath, e);
        } // end try catch

        // sanity check
        if (documentXComponent == null) {
            throw new IOException(
                    "Remote OpenOffice couldn't open template file "
                            + documentPath);
        } // end if

        // create the file and store the pdf
        final File resultFile = File.createTempFile("document", ".pdf");

        log("Storing to file: " + resultFile.getCanonicalPath());

        final XModel xModel = (XModel) UnoRuntime.queryInterface(XModel.class,
                documentXComponent);

        final XFrame xFrame = xModel.getCurrentController().getFrame();
        xFrame.activate();

        final XStorable xStorable = (XStorable) UnoRuntime.queryInterface(
                XStorable.class, documentXComponent);
        final PropertyValue[] storeProps = new PropertyValue[2];
        storeProps[0] = new PropertyValue();
        storeProps[0].Name = "FilterName";
        // storeProps[0].Value = "MS Word 97";
        // storeProps[0].Value = "pdf_Portable_Document_Format";
        // storeProps[0].Value = "writer_web_pdf_Export";
        storeProps[0].Value = "writer_pdf_Export";
        // storeProps[0].Value = "Rich Text Format";
        storeProps[1] = new PropertyValue();
        storeProps[1].Name = "CompressionMode";
        storeProps[1].Value = "1";

        xStorable.storeToURL("file:///" + resultFile.getCanonicalPath(),
                storeProps);

        log("Created File: " + resultFile.getAbsolutePath());

        // It is a full featured office document.
        // Try to use close mechanism instead of a hard dispose().
        // But maybe such service is not available on this model.
        com.sun.star.util.XCloseable xCloseable = (com.sun.star.util.XCloseable) UnoRuntime
                .queryInterface(com.sun.star.util.XCloseable.class, xModel);

        if (xCloseable != null) {
            try {
                xCloseable.close(true);
            } catch (com.sun.star.util.CloseVetoException exCloseVeto) {
                exCloseVeto.printStackTrace();
            }
        } else {
            // If close is not supported by this model - try to dispose it.
            // But if the model disagree with a reset request for the modify
            // state we shouldn't do so. Otherwise some strange things can
            // happen.
            final com.sun.star.lang.XComponent xDisposeable = (com.sun.star.lang.XComponent) UnoRuntime
                    .queryInterface(com.sun.star.lang.XComponent.class, xModel);
            xDisposeable.dispose();
        } // end if else

        // poor man's pdf validity check
        if (resultFile.length() < expectedSize.intValue()) {
            throw new Exception("Invalid PDF detected: "
                    + resultFile.getAbsolutePath());
        } // end if

        return resultFile;
    } // end saveDocumentAsPDF(String,Integer)

    /**
     * Prepends the timestamp
     * 
     * @param message
     */
    public void log(final String message) {
        System.out.println(new StringBuffer((new Date()).toString()).append(
                " - ").append(message).toString());
    } // end log(String)

    /**
     * Main method
     * 
     * @param args
     */
    public static void main(final String[] args) {
        // sanity check
        if (args.length != 4) {
            printUsage();
            return;
        } // end if

        final String documentPath = args[0];
        final Integer maxLoops = new Integer(args[1]);
        final Integer expectedSize = new Integer(args[2]);
        final String oOoPath = args[3];

        final TestSaveAsPDF testSaveAsPDF;
        try {
            testSaveAsPDF = new TestSaveAsPDF(new File(oOoPath));
        } catch (Exception e) {
            System.out.println("Could not start OOo");
            e.printStackTrace();
            return;
        } // end try catch

        System.out.println("Using document: " + documentPath);
        for (int loopCount = 0; loopCount < maxLoops.intValue(); loopCount++) {
            try {
                final File file = testSaveAsPDF.saveDocumentAsPDF(documentPath,
                        expectedSize);
            } catch (final Exception e) {
                e.printStackTrace();
            } // end try catch
            System.out.println("---------- done with iteration ----------");
        } // end for

        System.out.println("================= done ================");
    } // end main(String[])

    public static void printUsage() {
        System.out
                .println("Usage: java "
                        + TestSaveAsPDF.class.getName()
                        + " <document> <max_loops> <expected_size_in_bytes> <OOo_path>");
        System.out
                .println("Note: Don't forget your $LD_LIBRARY_PATH needs to point to open office");
    } // end printUsage()

} // end TestSaveAsPDF
